@@ -32,8 +32,8 @@ namespace clang {
 /// symlinks to files.  Its advantages are that it is dense and more efficient
 /// to create and process than a directory of symlinks.
 class HeaderMap {
-  HeaderMap(const HeaderMap &) LLVM_DELETED_FUNCTION;
-  void operator=(const HeaderMap &) LLVM_DELETED_FUNCTION;
+  HeaderMap(const HeaderMap &) = delete;
+  void operator=(const HeaderMap &) = delete;
 
   std::unique_ptr<const llvm::MemoryBuffer> FileBuffer;
   bool NeedsBSwap;
