@@ -1202,9 +1202,9 @@ namespace dr497 { // dr497: yes
     struct S {
       mutable int i;
     };
-    const S cs; // expected-error {{default initialization}} expected-note {{add an explicit initializer}}
+    const S cs; // expected-error {{default initialization}}
     int S::*pm = &S::i;
-    cs.*pm = 88;
+    cs.*pm = 88; // expected-error {{not assignable}}
   }
 
   void after() {
